// Importing the required D3 modules
import {
  select,
  csv,
  pie,
  arc,
  scaleOrdinal,
  schemeCategory10,
} from 'd3';

// Selecting the SVG element from the DOM
const svg = select('svg');
// Getting the width and height of the SVG element
const width = +svg.attr('width');
const height = +svg.attr('height');

// Function to format numbers for display
const numberFormat = (number) =>
  Math.abs(Number(number)) + ' hp ';

// Render function to create the pie chart
const render = (data) => {
  // Creating the pie chart data
  const pieData = pie()
    .sort(null) // Disable sorting of pie slices
    .value((d) => d.hp)(data); // Using 'hp' as the value accessor for the pie chart

  // Creating a scale for colors
  const colors = scaleOrdinal(schemeCategory10);

  // Creating an arc generator
  const segments = arc()
    .innerRadius(0) // Inner radius of the pie chart
    .outerRadius(180) // Outer radius of the pie chart
    .padAngle(0) // Spacing between slices
    .padRadius(0); // Padding between slices and the center

  // Adding the pie slices to the SVG
  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  sections
    .enter()
    .append('path')
    .attr('d', segments) // Setting the path of each pie slice
    .attr('fill', (d) => colors(d.data.hp)) // Setting the fill color based on 'hp' value
    .on('mouseover', (d) => console.log(d.data.hp)); // Log 'hp' value on mouseover

  // Creating legends for the pie chart
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);

  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 30})`; // Positioning each legend item
    });

  // Adding rectangles for color indicators in legends
  legend
    .append('rect')
    .attr('width', 22)
    .attr('height', 25)
    .attr('fill', (d) => colors(d.data.hp));

  // Adding text for model names in legends
  legend
    .append('text')
    .attr('x', 28)
    .attr('y', 18)
    .attr('class', 'legend_text')
    .text((d) => d.data.model);

  // Adding text for hp values in legends
  legend
    .append('text')
    .attr('x', 260)
    .attr('y', 20)
    .attr('class', 'legend_value')
    .text((d) => numberFormat(d.data.hp));
};

// URL of the dataset
const csvUrl =
  'https://gist.githubusercontent.com/NandithaSura/784eef9bc929bc7f7eae9627b8fd74d4/raw/2654abeb912711ee2a3741ecad4d03240fa850bb/ice7.csv';

// Load the data and display basic cost of Netflix
csv(csvUrl).then((csvData) => {
  csvData.sort((data1, data2) => data2.hp - data1.hp); // Sorting data based on 'hp'
  if (csvData) {
    render(csvData.slice(0, 12)); // Render the pie chart with top 12 data points
  }
});
